-- ===========================================================================
--	Single Player Create Game w/ Advanced Options
-- ===========================================================================
include("InstanceManager");
include("PlayerSetupLogic");
include("Civ6Common");

-- ===========================================================================
-- ===========================================================================

local m_NonLocalPlayerSlotManager	:table = InstanceManager:new("NonLocalPlayerSlotInstance", "Root", Controls.NonLocalPlayersSlotStack);
local m_CivManager					:table = InstanceManager:new("CivilizationInstance", "Root", Controls.CivilizationStack);
local m_CivDetailManager			:table = InstanceManager:new("CivilizationDetailInstance", "Root", Controls.DetailStack);
local m_singlePlayerID				:number = 0;			-- The player ID of the human player in singleplayer.
local m_AdvancedMode				:boolean = false;
local m_CivSetup					:boolean = false;

-- ===========================================================================
-- Input Handler
-- ===========================================================================
function OnInputHandler( pInputStruct:table )
	local uiMsg = pInputStruct:GetMessageType();
	if uiMsg == KeyEvents.KeyUp then
		local key:number = pInputStruct:GetKey();
		if key == Keys.VK_ESCAPE then
			OnBackButton();
		end
	end
	return true;
end

-- ===========================================================================
function CreatePulldownDriver(o, parameter, c)
	local driver = {
		Control = c,
		UpdateValue = function(value)
			local button = c:GetButton();
			button:SetText( value and value.Name or nil);
		end,
		UpdateValues = function(values)
			c:ClearEntries();

			for i,v in ipairs(values) do
				local entry = {};
				c:BuildEntry( "InstanceOne", entry );
				entry.Button:SetText(v.Name);
				entry.Button:SetToolTipString(v.Description);

				entry.Button:RegisterCallback(Mouse.eLClick, function()
					o:SetParameterValue(parameter, v);
					Network.BroadcastGameConfig();
				end);
			end
			c:CalculateInternals();
		end,
		SetEnabled = function(enabled)
			c:SetDisabled(not enabled);
		end,
		SetVisible = nil,	-- Never hide the basic pulldown.
		Destroy = nil,		-- It's a fixed control, no need to delete.
	};
	
	return driver;	
end

-- ===========================================================================
-- Override parameter behavior for basic setup screen.
g_ParameterFactories["GameDifficulty"] = function(o, parameter)

	local drivers = {};
	-- Basic setup version.
	-- Use an explicit table.
	table.insert(drivers, CreatePulldownDriver(o, parameter, Controls.CreateGame_GameDifficulty));

	-- Advanced setup version.
	-- Create the parameter dynamically like we normally would...
	table.insert(drivers, GameParameters_UI_DefaultCreateParameterDriver(o, parameter));

	return drivers;
end

-- ===========================================================================
g_ParameterFactories["GameSpeeds"] = function(o, parameter)

	local drivers = {};
	-- Basic setup version.
	-- Use an explicit table.
	table.insert(drivers, CreatePulldownDriver(o, parameter, Controls.CreateGame_SpeedPulldown));

	-- Advanced setup version.
	-- Create the parameter dynamically like we normally would...
	table.insert(drivers, GameParameters_UI_DefaultCreateParameterDriver(o, parameter));

	return drivers;
end

-- ===========================================================================
g_ParameterFactories["Map"] = function(o, parameter)

	local drivers = {};
	-- Basic setup version.
	-- Use an explicit table.
	table.insert(drivers, CreatePulldownDriver(o, parameter, Controls.CreateGame_MapType));

	-- Advanced setup version.
	-- Create the parameter dynamically like we normally would...
	table.insert(drivers, GameParameters_UI_DefaultCreateParameterDriver(o, parameter));

	return drivers;
end

-- ===========================================================================
g_ParameterFactories["MapSize"] = function(o, parameter)

	local drivers = {};
	-- Basic setup version.
	-- Use an explicit table.
	table.insert(drivers, CreatePulldownDriver(o, parameter, Controls.CreateGame_MapSize));

	-- Advanced setup version.
	-- Create the parameter dynamically like we normally would...
	table.insert(drivers, GameParameters_UI_DefaultCreateParameterDriver(o, parameter));

	return drivers;
end

-- ===========================================================================
-- Remove player handler.
function RemovePlayer(voidValue1, voidValue2, control)
	print("Removing Player " .. tonumber(voidValue1));
	local playerConfig = PlayerConfigurations[voidValue1];
	playerConfig:SetLeaderTypeName(nil);
	
	GameConfiguration.RemovePlayer(voidValue1);

	LuaEvents.GameSetup_PlayerCountChanged();
	GameSetup_RefreshParameters();
end

-- ===========================================================================
-- Add UI entries for all the players.  This does not set the
-- UI values of the player.
-- ===========================================================================
function RefreshPlayerSlots()

	RebuildPlayerParameters();
	m_NonLocalPlayerSlotManager:ResetInstances();

	local player_ids = GameConfiguration.GetParticipatingPlayerIDs();

	local minPlayers = MapConfiguration.GetMinMajorPlayers() or 2;
	local maxPlayers = MapConfiguration.GetMaxMajorPlayers() or 2;
	local can_remove = #player_ids > minPlayers;
	local can_add = #player_ids < maxPlayers;

	Controls.AddAIButton:SetHide(not can_add);

	print("There are " .. #player_ids .. " participating players.");

	--Controls.BasicTooltipContainer:DestroyAllChildren();
	--Controls.BasicPlacardContainer:DestroyAllChildren();
	--Controls.AdvancedTooltipContainer:DestroyAllChildren();

	local basicTooltip	:table = {};	
	--ContextPtr:BuildInstanceForControl( "CivToolTip", basicTooltip, Controls.BasicTooltipContainer );
	local basicPlacard	:table = {};
	--ContextPtr:BuildInstanceForControl( "LeaderPlacard", basicPlacard, Controls.BasicPlacardContainer );

	local basicTooltipData : table = {
		InfoStack			= basicTooltip.InfoStack,
		InfoScrollPanel		= basicTooltip.InfoScrollPanel;
		CivToolTipSlide		= basicTooltip.CivToolTipSlide;
		CivToolTipAlpha		= basicTooltip.CivToolTipAlpha;
		UniqueIconIM		= InstanceManager:new("IconInfoInstance",	"Top",	basicTooltip.InfoStack );		
		HeaderIconIM		= InstanceManager:new("IconInstance",		"Top",	basicTooltip.InfoStack );
		HeaderIM			= InstanceManager:new("HeaderInstance",		"Top",	basicTooltip.InfoStack );
		HasLeaderPlacard	= true;
		LeaderBG			= basicPlacard.LeaderBG;
		LeaderImage			= basicPlacard.LeaderImage;
		DummyImage			= basicPlacard.DummyImage;
		CivLeaderSlide		= basicPlacard.CivLeaderSlide;
		CivLeaderAlpha		= basicPlacard.CivLeaderAlpha;
	};

	local advancedTooltip	:table = {};
	--ContextPtr:BuildInstanceForControl( "CivToolTip", advancedTooltip, Controls.AdvancedTooltipContainer );

	local advancedTooltipData : table = {
		InfoStack			= advancedTooltip.InfoStack,
		InfoScrollPanel		= advancedTooltip.InfoScrollPanel;
		CivToolTipSlide		= advancedTooltip.CivToolTipSlide;
		CivToolTipAlpha		= advancedTooltip.CivToolTipAlpha;
		UniqueIconIM		= InstanceManager:new("IconInfoInstance",	"Top",	advancedTooltip.InfoStack );		
		HeaderIconIM		= InstanceManager:new("IconInstance",		"Top",	advancedTooltip.InfoStack );
		HeaderIM			= InstanceManager:new("HeaderInstance",		"Top",	advancedTooltip.InfoStack );
		HasLeaderPlacard	= false;
	};

	for i, player_id in ipairs(player_ids) do	
		if(m_singlePlayerID == player_id) then
			SetupLeaderPulldown(player_id, Controls, "Basic_LocalPlayerPulldown", "Basic_LocalPlayerCivIcon", "Basic_LocalPlayerLeaderIcon", basicTooltipData);
			SetupLeaderPulldown(player_id, Controls, "Advanced_LocalPlayerPulldown", "Advanced_LocalPlayerCivIcon", "Advanced_LocalPlayerLeaderIcon", advancedTooltipData);
		else
			local ui_instance = m_NonLocalPlayerSlotManager:GetInstance();
			
			-- Assign the Remove handler
			if(can_remove) then
				ui_instance.RemoveButton:SetVoid1(player_id);
				ui_instance.RemoveButton:RegisterCallback(Mouse.eLClick, RemovePlayer);
			end
			ui_instance.RemoveButton:SetHide(not can_remove);
			
			SetupLeaderPulldown(player_id, ui_instance,"PlayerPullDown",nil,nil,advancedTooltipData);
		end
	end

	Controls.NonLocalPlayersSlotStack:CalculateSize();
	Controls.NonLocalPlayersSlotStack:ReprocessAnchoring();
	Controls.NonLocalPlayersStack:CalculateSize();
	Controls.NonLocalPlayersStack:ReprocessAnchoring();
	
	Controls.NonLocalPlayersPanel:CalculateInternalSize();
	GameSetup_RefreshParameters();
end

-- ===========================================================================
function OnPlayerCountChanged()
	print("Player Count Changed");
	RefreshPlayerSlots();
end

-- ===========================================================================
function OnShow()
	BuildGameSetup();
	RefreshPlayerSlots();
	GameSetup_RefreshParameters();
	AutoSizeGridButton(Controls.DefaultButton,50,22,10,"H");
	AutoSizeGridButton(Controls.CloseButton,133,36,10,"H");
end

-- ===========================================================================
function OnHide()
	HideGameSetup();
	ReleasePlayerParameters();
end


-- ===========================================================================
-- Button Handlers
-- ===========================================================================

-- ===========================================================================
function OnAddAIButton()
	-- Search for an empty slot number and mark the slot as computer.
	-- Then dispatch the player count changed event.
	local iPlayer = 0;
	while(true) do
		local playerConfig = PlayerConfigurations[iPlayer];
		
		-- If we've reached the end of the line, exit.
		if(playerConfig == nil) then
			break;
		end

		-- Find a suitable slot to add the AI.
		if (playerConfig:GetSlotStatus() == SlotStatus.SS_CLOSED) then
			playerConfig:SetSlotStatus(SlotStatus.SS_COMPUTER);
			playerConfig:SetMajorCiv();

			LuaEvents.GameSetup_PlayerCountChanged();
			GameSetup_RefreshParameters();
			Controls.NonLocalPlayersPanel:CalculateSize();
			break;
		end

		-- Increment the AI, this assumes that either player config will hit nil 
		-- or we'll reach a suitable slot.
		iPlayer = iPlayer + 1;
	end
end

-- ===========================================================================
function OnAdvancedSetup()
	Controls.CreateGameWindow:SetHide(true);
	Controls.AdvancedOptionsWindow:SetHide(false);
	m_AdvancedMode = true;
end

function OnCivSelection()
	Controls.CreateGameWindow:SetHide(true);
	Controls.AdvancedOptionsWindow:SetHide(true);
	Controls.StartButton:SetHide(true);
	Controls.CivSelectionScreen:SetHide(false);
	m_CivSetup = true;

	ListCivilizations()
end
-- ===========================================================================
function OnDefaultButton()
	print("Reseting Setup Parameters");
	g_Refreshing = true;
	g_GameParameters:ResetDefaults();
	GameConfiguration.RegenerateSeeds();
    g_GameParameters:FullRefresh();
	ResetPlayerParameters();
	g_Refreshing = false;
	if(g_NeedsAdditionalRefresh) then
		g_NeedsAdditionalRefresh = false;
		print("Refreshing again, to be sure.")
		return GameSetup_RefreshParameters();
	end

end

-- ===========================================================================
function OnStartButton()
	-- Is WorldBuilder active?
	if (GameConfiguration.IsWorldBuilder()) then
		UI.SetWorldRenderView( WorldRenderView.VIEW_2D );
		UI.PlaySound("Set_View_2D");
		Network.HostGame(ServerType.SERVER_TYPE_NONE);
		
	else
		-- No, start a normal game
		UI.PlaySound("Set_View_3D");
		Network.HostGame(ServerType.SERVER_TYPE_NONE);
	end
end



----------------------------------------------------------------    
function OnBackButton()
	if(m_CivSetup) then
		Controls.CreateGameWindow:SetHide(m_AdvancedMode);
		Controls.AdvancedOptionsWindow:SetHide(not m_AdvancedMode);
		Controls.StartButton:SetHide(false);
		Controls.CivSelectionScreen:SetHide(true);
		m_CivSetup = false;
		return
	end

	if(m_AdvancedMode) then
		Controls.CreateGameWindow:SetHide(false);
		Controls.AdvancedOptionsWindow:SetHide(true);
		m_AdvancedMode = false;		
	else
		UIManager:DequeuePopup( ContextPtr );
	end
end

----------------------------------------------------------------    
-- ===========================================================================
--	Handle Window Sizing
-- ===========================================================================

function Resize()
	local screenX, screenY:number = UIManager:GetScreenSizeVal();
	local hideLogo = true;
	if(screenY >= Controls.MainWindow:GetSizeY() + (Controls.LogoContainer:GetSizeY()+ Controls.LogoContainer:GetOffsetY())*2) then
		hideLogo = false;
	end
	Controls.LogoContainer:SetHide(hideLogo);
	Controls.MainGrid:ReprocessAnchoring();
end

-- ===========================================================================
function OnUpdateUI( type:number, tag:string, iData1:number, iData2:number, strData1:string )   
  if type == SystemUpdateUI.ScreenResize then
    Resize();
  end
end

-- ===========================================================================
function OnBeforeMultiplayerInviteProcessing()
	-- We're about to process a game invite.  Get off the popup stack before we accidently break the invite!
	UIManager:DequeuePopup( ContextPtr );
end

-- ===========================================================================
--
-- ===========================================================================
function Initialize()

	ContextPtr:SetInputHandler( OnInputHandler, true );
	ContextPtr:SetShowHandler( OnShow );
	ContextPtr:SetHideHandler( OnHide );

	Controls.AddAIButton:RegisterCallback( Mouse.eLClick, OnAddAIButton );
	Controls.AddAIButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.AdvancedSetupButton:RegisterCallback( Mouse.eLClick, OnAdvancedSetup );
	Controls.AdvancedSetupButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.SelectCivButton:RegisterCallback( Mouse.eLClick, OnCivSelection );
	Controls.SelectCivButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.DefaultButton:RegisterCallback( Mouse.eLClick, OnDefaultButton);
	Controls.DefaultButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.StartButton:RegisterCallback( Mouse.eLClick, OnStartButton );
	Controls.StartButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnBackButton );
	Controls.CloseButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);

	LuaEvents.GameSetup_PlayerCountChanged.Add(OnPlayerCountChanged);

	Events.SystemUpdateUI.Add( OnUpdateUI );
	Events.BeforeMultiplayerInviteProcessing.Add( OnBeforeMultiplayerInviteProcessing );
	Resize();
end
Initialize();

-- ===========================================================================
-- Sukritact's True Alternate Leaders
-- ===========================================================================
function shallowcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

function Civ_OnMouseOver(tCivData_Org, tData)
	--print(tCivData.Name)

	local tCivData = shallowcopy(tCivData_Org)

	m_CivDetailManager:ResetInstances()

	-- Make leader info come before uniques
	if tData then
		table.insert(tCivData, 2, tData[1])
		tCivData[tData[1].Type] = true
		tCivData.HasLeader = true
	end

	for iNum, tRow in ipairs(tCivData) do
		local tInstance = m_CivDetailManager:GetInstance()

		if tRow.IsBuilding then
			tInstance.Unique_Icon:SetSizeVal(38,38);
			tInstance.Unique_Icon:ReprocessAnchoring();
		end

		tInstance.Unique_Icon:SetIcon(tRow.Icon)
		tInstance.Unique_Name:SetText(Locale.ToUpper(tRow.Name))
		tInstance.Unique_Desc:SetText(tRow.Desc)

		tInstance.LabelStack:CalculateSize();
		local iStackY = tInstance.LabelStack:GetSizeY()
		iStackY = iStackY + 30

		tInstance.Root:SetSizeY(iStackY)
		tInstance.LabelStack:ReprocessAnchoring();
		tInstance.Root:ReprocessAnchoring();
	end

	if tData then
		for iNum, tItem in ipairs(tData) do
			if not tCivData[tItem.Type] then

				local tInstance = m_CivDetailManager:GetInstance()

				if tItem.IsBuilding then
					tInstance.Unique_Icon:SetSizeVal(38,38);
					tInstance.Unique_Icon:ReprocessAnchoring();
				end

				tInstance.Unique_Icon:SetIcon(tItem.Icon)
				tInstance.Unique_Name:SetText(Locale.ToUpper(tItem.Name))
				tInstance.Unique_Desc:SetText(tItem.Desc)

				tInstance.LabelStack:CalculateSize();
				local iStackY = tInstance.LabelStack:GetSizeY()
				iStackY = iStackY + 30

				tInstance.Root:SetSizeY(iStackY)
				tInstance.LabelStack:ReprocessAnchoring();
				tInstance.Root:ReprocessAnchoring();

			end
		end
	end

	Controls.DetailStack:CalculateSize();
	Controls.DetailStack:ReprocessAnchoring();
	Controls.DetailScrollPanel:CalculateInternalSize();
end

function ListCivilizations()

	m_CivManager:ResetInstances()
	Controls.CivBackButtonFrame:SetHide(true)
	Controls.CivRandomButtonFrame:SetHide(false)
	m_isTraitsFullDescriptions = true

	local tCivilizations = {}

	local sQueryCivs = "SELECT CivilizationIcon, CivilizationType, LeaderType, LeaderIcon, LeaderName, CivilizationName, LeaderAbilityName, LeaderAbilityDescription, LeaderAbilityIcon, CivilizationAbilityName, CivilizationAbilityDescription, CivilizationAbilityIcon from Players ORDER BY CivilizationType";
	local tQueryCivs = DB.ConfigurationQuery(sQueryCivs);
	local sQueryUniques = "SELECT Type, Name, Description, Icon from PlayerItems where LeaderType = ? ORDER BY SortIndex";
	local bFirst = true

	for iNum, tRow in ipairs(tQueryCivs) do
		if not tCivilizations[tRow.CivilizationName] then

			local tData = {}
			local tCivUniqueAbilities, tCivUniqueUnits, tCivUniqueBuildings = GetCivilizationUniqueTraits(tRow.CivilizationType);
			local tOtherUniques = DB.ConfigurationQuery(sQueryUniques, tRow.LeaderType);

			-------------------------------------
			-- UA
			-------------------------------------
			tData[1] = {
				Type = tRow.CivilizationType,
				Icon = tRow.CivilizationIcon,
				Name = Locale.Lookup(tRow.CivilizationName)  .. " - " ..  Locale.Lookup(tRow.CivilizationAbilityName),
				Desc = Locale.Lookup(tRow.CivilizationAbilityDescription)
			}

			tData[tRow.CivilizationType] = true
			-------------------------------------
			-- UU
			-------------------------------------
			if tCivUniqueUnits[1] then
				tData[2] = {
					Type = tCivUniqueUnits[1].Type,
					Icon = "ICON_"..tCivUniqueUnits[1].Type,
					Name = Locale.Lookup(tCivUniqueUnits[1].Name),
					Desc = Locale.Lookup(tCivUniqueUnits[1].Description),
				}

				tData[tCivUniqueUnits[1].Type] = true
			else
				for _, tItem in ipairs(tOtherUniques) do
					if string.find(tItem.Type, "UNIT") then
						tData[2] = {
							Type = tItem.Type,
							Icon = tItem.Icon,
							Name = Locale.Lookup(tItem.Name),
							Desc = Locale.Lookup(tItem.Description),
						}

						tData[tItem.Type] = true						
						break
					end
				end
			end
			-------------------------------------
			-- UB
			-------------------------------------
			if tCivUniqueBuildings[1] then
				tData[3] = {
					Type = tCivUniqueBuildings[1].Type,
					Icon = "ICON_"..tCivUniqueBuildings[1].Type,
					Name = Locale.Lookup(tCivUniqueBuildings[1].Name),
					Desc = Locale.Lookup(tCivUniqueBuildings[1].Description),
					IsBuilding = true,
				}

				tData[tCivUniqueBuildings[1].Type] = true
			else
				for _, tItem in ipairs(tOtherUniques) do
					if (string.find(tItem.Type, "BUILDING") or string.find(tItem.Type, "IMPROVEMENT") or string.find(tItem.Type, "DISTRICT")) then
						tData[3] = {
							Type = tItem.Type,
							Icon = tItem.Icon,
							Name = Locale.Lookup(tItem.Name),
							Desc = Locale.Lookup(tItem.Description),
							IsBuilding = true,
						}

						tData[tItem.Type] = true		
						break
					end
				end
			end

			local tInstance = m_CivManager:GetInstance()
			tInstance.Root:SetSizeY(110)
			tInstance.CivDesc:SetHide(true)
			tInstance.CivInstanceDataStack:SetHide(false)

			tInstance.CivIcon:SetColor(0x40ffffff)
			tInstance.CivIcon:SetIcon(tData[1].Icon)
			tInstance.CivName:SetText(Locale.ToUpper(tData[1].Name))

			tInstance.Unique_1_Icon:SetIcon(tData[2].Icon)
			tInstance.Unique_1_Name:SetText(tData[2].Name)
			
			tInstance.Unique_2_Icon:SetSizeVal(38,38);
			tInstance.Unique_2_Icon:ReprocessAnchoring();
			tInstance.Unique_2_Icon:SetIcon(tData[3].Icon)
			tInstance.Unique_2_Name:SetText(tData[3].Name)

			tCivilizations[tRow.CivilizationName] = true
			tInstance.Button:RegisterCallback(Mouse.eMouseEnter,
				function()
					Civ_OnMouseOver(tData); 
				end
			)
			tInstance.Button:RegisterCallback(Mouse.eLClick,
				function()
					ListLeaders(tRow.CivilizationType, tData); 
				end
			)

			if bFirst then 			
				Civ_OnMouseOver(tData)
				bFirst = false
			end
		end
	end

	m_isTraitsFullDescriptions = false
	Controls.CivilizationStack:CalculateSize();
	Controls.CivilizationStack:ReprocessAnchoring();
	Controls.CivilizationScrollPanel:CalculateInternalSize();
end

Controls.CivBackButton:RegisterCallback(Mouse.eLClick,
	function()
		ListCivilizations();
	end
)
tInstancesToRemove = {}

function ListLeaders(sCivilizationType, tCivData)
	
	m_CivManager:ResetInstances()
	Controls.CivBackButtonFrame:SetHide(false)
	Controls.CivRandomButtonFrame:SetHide(true)

	local sQueryCivs = "SELECT CivilizationIcon, CivilizationType, LeaderType, LeaderIcon, LeaderName, CivilizationName, LeaderAbilityName, LeaderAbilityDescription, LeaderAbilityIcon, CivilizationAbilityName, CivilizationAbilityDescription, CivilizationAbilityIcon from Players where CivilizationType = ? ORDER BY LeaderType";
	local tQueryCivs = DB.ConfigurationQuery(sQueryCivs, sCivilizationType);
	local sQueryUniques = "SELECT Type, Name, Description, Icon from PlayerItems where LeaderType = ? ORDER BY SortIndex";
	local bFirst = true

	for _, tRow in ipairs(tQueryCivs) do

		tRow.Value = tRow.LeaderType
		local tData = {}
		local tOtherUniques = DB.ConfigurationQuery(sQueryUniques, tRow.LeaderType);

		-------------------------------------
		-- UA
		-------------------------------------
		tData[1] = {
			Type = tRow.LeaderType,
			Icon = tRow.LeaderIcon,
			Name = Locale.Lookup(tRow.LeaderName)  .. " - " ..  Locale.Lookup(tRow.LeaderAbilityName),
			Desc = Locale.Lookup(tRow.LeaderAbilityDescription)
		}
		-------------------------------------
		-- UU/UBs
		-------------------------------------
		local iNum = 2;

		for _, tItem in ipairs(tOtherUniques) do
			tData[iNum] = {
				Type = tItem.Type,
				Icon = tItem.Icon,
				Name = Locale.Lookup(tItem.Name),
				Desc = Locale.Lookup(tItem.Description),
				IsBuilding = (string.find(tItem.Type, "BUILDING") or string.find(tItem.Type, "IMPROVEMENT") or string.find(tItem.Type, "DISTRICT"))
			}		
			
			iNum = iNum + 1
		end


		-- for iNum, tItem in ipairs(tData) do
		-- 	if tCivData[tItem.Type] then
		-- 		table.remove(tData, iNum)
		-- 	end
		-- end

		local tInstance = m_CivManager:GetInstance()
		tInstance.Root:SetSizeY(150)
		tInstance.CivDesc:SetHide(false)
		tInstance.CivInstanceDataStack:SetHide(true)

		tInstance.CivIcon:SetIcon(tData[1].Icon)
		tInstance.CivIcon:SetColor(0xffffffff)
		tInstance.CivName:SetText(Locale.ToUpper(tData[1].Name))
		tInstance.CivDesc:SetText(tData[1].Desc)

		tInstance.Button:RegisterCallback(Mouse.eMouseEnter,
			function()
				Civ_OnMouseOver(tCivData, tData); 
			end
		)
		tInstance.Button:RegisterCallback(Mouse.eLClick,
			function()
				local parameters = GetPlayerParameters(m_singlePlayerID);
				if(parameters == nil) then
					parameters = CreatePlayerParameters(m_singlePlayerID);
				end

				local parameter = parameters.Parameters["PlayerLeader"];
				parameters:SetParameterValue(parameter, tRow);

				local tInfo = GetPlayerInfo("StandardPlayers", tRow.LeaderType)
				local sTooltip = GenerateToolTipFromPlayerInfo(tInfo)
				local sCivName = Locale.Lookup(tRow.CivilizationName)  .. " - " ..  Locale.Lookup(tRow.LeaderName)

				Controls.Preview_LocalPlayerCivIcon:SetIcon(tRow.CivilizationIcon)
				Controls.Preview_LocalPlayerLeaderIcon:SetIcon(tRow.LeaderIcon)
				Controls.PreviewCivName:SetText(sCivName)
				Controls.PreviewBox:SetToolTipString(sTooltip)

				OnBackButton();
			end
		)

		if bFirst then 			
			Civ_OnMouseOver(tCivData, tData)
			bFirst = false
		end		
	end

	m_isTraitsFullDescriptions = false
	Controls.CivilizationStack:CalculateSize();
	Controls.CivilizationStack:ReprocessAnchoring();
	Controls.CivilizationScrollPanel:CalculateInternalSize();
end

Controls.CivRandomButton:RegisterCallback(Mouse.eLClick,
	function()
		local parameters = GetPlayerParameters(m_singlePlayerID);
		if(parameters == nil) then
			parameters = CreatePlayerParameters(m_singlePlayerID);
		end

		local parameter = parameters.Parameters["PlayerLeader"];
		parameters:SetParameterValue(parameter, "Random");

		local sCivName = Locale.Lookup("LOC_RANDOM_LEADER")

		Controls.Preview_LocalPlayerCivIcon:SetIcon("ICON_CIVILIZATION_UNKNOWN")
		Controls.Preview_LocalPlayerLeaderIcon:SetIcon("ICON_LEADER_DEFAULT")
		Controls.PreviewCivName:SetText(sCivName)
		Controls.PreviewBox:EnableToolTip(false)

		OnBackButton();
	end
)